@@ -10,6 +10,7 @@ import re
 from Bio.Seq import Seq
 
 from TileSeqMut import posterior
+pd.options.mode.chained_assignment = None  # default='warn'
 # import posterior
 
 class MutParser(object):
@@ -26,7 +27,6 @@ class MutParser(object):
         self._cds = Seq(cds_seq)
         self._start_pos = full_seq.cds_start.values.item() # 1 posisiton
         self._end_pos = full_seq.cds_end.values.item()
-
         self._tile_begins = tile_s
         self._tile_ends = tile_e
         # Note all the position in this df are 1-based
@@ -141,7 +141,14 @@ class MutParser(object):
         merged_df = [snp_df, delins_df]
         merged_df = pd.concat(merged_df)
         merged_df = merged_df.sort_values(by="pos")
-
+        merged_df = merged_df.groupby("pos").first().reset_index()
+        # build df to track how many mutations were rejected 
+        track_df = merged_df[["pos", "m_r1", "m_r2"]]
+        track_df["pos"] = track_df["pos"] - self._start_pos + 1
+        # adjust positions in the df, the postions are template pos not cds pos
+        track_df[["m_r1", "m_r2"]] = track_df[["m_r1", "m_r2"]].where(~track_df[["m_r1", "m_r2"]].notna(), 1)
+        track_df[["m_r1", "m_r2"]] = track_df[["m_r1", "m_r2"]].fillna(0)
+        track_df["m_both"] = track_df["m_r1"] + track_df["m_r2"] - 1
         # group mutations based on positions
         # any mutations that are within 3bp are grouped together
         n = 3
@@ -154,6 +161,20 @@ class MutParser(object):
                                                                        self._posteriorQC)
         final_mut = list(set(pos_df.m.tolist()))
         final_mut.sort()
+        if final_mut != []:
+            final_df = pd.DataFrame([sub.split("|") for sub in final_mut])
+            # give the position col a name
+            final_df = final_df.rename(columns=str).rename(columns={"0": "pos", "1": "passed"})
+            final_df = final_df[["pos", "passed"]]
+            final_df["pos"] = final_df["pos"].astype(int)
+            final_df["pos"] = final_df["pos"] - self._start_pos + 1
+            # merge final df with track df to see how many mutations passed filter on each/both reads
+            merged_track_df = pd.merge(track_df, final_df, how="left", on="pos")
+            merged_track_df["passed"] = merged_track_df["passed"].where(~merged_track_df["passed"].notna(), 1)
+            merged_track_df["passed"] = merged_track_df["passed"].fillna(0)
+        else:
+            merged_track_df = track_df
+            merged_track_df["passed"] = 0
         hgvs_r1_clusters, outside_mut_r1 = [], []
         hgvs_r2_clusters, outside_mut_r2 = [], []
         if not clustered_r1.empty:
@@ -174,7 +195,7 @@ class MutParser(object):
         else:
             hgvs, outside_mut = [], []
 
-        return hgvs, outside_mut, all_df, hgvs_r1_clusters, hgvs_r2_clusters
+        return hgvs, outside_mut, all_df, hgvs_r1_clusters, hgvs_r2_clusters, merged_track_df
 
     def _parse_cigar_mdz(self, cigar, mdz_raw, ref, read, pos, qual):
         """
